﻿/**
 * @file	sdl2event.cpp
 * @brief	コンソール アプリケーションのエントリ ポイントを定義します。
 */

#include "stdafx.h"
#include <string>
#include <vector>
#include "sdl2window.hpp"

#ifdef _WIN32
#pragma comment(lib, "SDL2.lib")
#pragma comment(lib, "SDL2main.lib")
#endif	// _WIN32

/**
 * メイン
 * @param[in] argc 引数
 * @param[in] argv 引数
 * @return リザルト コード
 */
#ifdef _WIN32
int SDL_main(int argc, char *argv[])
#else	// _WIN32
int main(int argc, char *argv[])
#endif	// _WIN32
{
	Sdl2Window win;
	if (win.Create())
	{
		win.AddText("Hello World!\n");

		SDL_Init(SDL_INIT_JOYSTICK);
		const int nJoysticks = SDL_NumJoysticks();
		win.AddFormat("%d joystick(s) found.\n", nJoysticks);

		std::vector<SDL_Joystick*> joysticks;
		for (int i = 0; i < nJoysticks; i++)
		{
			SDL_Joystick* pJoystick = SDL_JoystickOpen(i);
			if (pJoystick)
			{
				joysticks.push_back(pJoystick);
				win.AddFormat(" - %d: %s - %d axes, %d balls, %d hats, %d buttons\n", i,
					SDL_JoystickName(pJoystick),
					SDL_JoystickNumAxes(pJoystick),
					SDL_JoystickNumBalls(pJoystick),
					SDL_JoystickNumHats(pJoystick),
					SDL_JoystickNumButtons(pJoystick));
			}
		}

		SDL_Init(SDL_INIT_AUDIO);
		const int nAudioDevices = SDL_GetNumAudioDevices(0);
		win.AddFormat("Audio %d device(s) found.\n", nAudioDevices);

		for (int i = 0; i < nAudioDevices; i++)
		{
			win.AddFormat(" - %d: %s\n", i, SDL_GetAudioDeviceName(i, 0));
		}

		win.AddFormat("Press power key to exit.\n");

		/* メインループ */
		bool bDone = false;
		while (!bDone)
		{
			SDL_Event e;
			if (SDL_PollEvent(&e))
			{
				switch (e.type)
				{
					case SDL_KEYDOWN:
						win.AddFormat("type:SDL_KEYDOWN, scancode:%d, sym:%08X\n", e.key.keysym.scancode, e.key.keysym.sym);
						break;

					case SDL_KEYUP:
						win.AddFormat("type:SDL_KEYUP, scancode:%d, sym:%08X\n", e.key.keysym.scancode, e.key.keysym.sym);
						bDone = (e.key.keysym.scancode == SDL_SCANCODE_POWER);
						break;

					case SDL_JOYAXISMOTION:
						win.AddFormat("type:SDL_JOYAXISMOTION, which:%d, axis:%d, value:%d\n", e.jaxis.which, e.jaxis.axis, e.jaxis.value);
						break;

					case SDL_JOYHATMOTION:
						{
							struct HatBit
							{
								int bit;
								const char* name;
							};
							static const HatBit s_hatbit[4] =
							{
								{SDL_HAT_UP,	"SDL_HAT_UP"},
								{SDL_HAT_RIGHT, "SDL_HAT_RIGHT"},
								{SDL_HAT_DOWN,	"SDL_HAT_DOWN"},
								{SDL_HAT_LEFT,	"SDL_HAT_LEFT"},
							};

							std::string info;
							for (unsigned int i = 0; i < 4; i++)
							{
								if (e.jhat.value & s_hatbit[i].bit)
								{
									if (!info.empty())
									{
										info += ", ";
									}
									info += s_hatbit[i].name;
								}
							}
							if (info.empty())
							{
								info += "SDL_HAT_CENTERED";
							}
							win.AddFormat("type:SDL_JOYHATMOTION, which:%d, axis:%d, value:%d(%s)\n", e.jhat.which, e.jhat.hat, e.jhat.value, info.c_str());
						}
						break;

					case SDL_JOYBUTTONDOWN:
						win.AddFormat("type:SDL_JOYBUTTONDOWN, which:%d, button:%d\n", e.jbutton.which, e.jbutton.button);
						break;

					case SDL_JOYBUTTONUP:
						win.AddFormat("type:SDL_JOYBUTTONUP, which:%d, button:%d\n", e.jbutton.which, e.jbutton.button);
						break;

					case SDL_QUIT:
						bDone = true;
						break;
				}
			}
			else
			{
				SDL_Delay(1);
			}
		}

		for (std::vector<SDL_Joystick*>::iterator it = joysticks.begin(); it != joysticks.end(); ++it)
		{
			SDL_JoystickClose(*it);
		}

		win.Destroy();
	}

	return 0;
}

